###
## SecClabCa
## Copyright (C) 2003 SecClab group (mail. lfp70@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public Licence as published by
# the Free Software Foundation; eihter version 2 of the License, or
# (at your option) any later version.
#
# This program is distributes in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##


import os
import string
import sys
import popen2
import time

if sys.platform[:3] == "win":
    import process
else:
    import fcntl
    import FCNTL
    
import select
import shutil
import traceback
from ConfigParser import ConfigParser
from ConfigParser import DuplicateSectionError
import types

from OpenSSLbin import OPENSSL_BIN


#sys.path=sys.path+[os.path.join('/usr/share/zope/','lib','python'),'/usr/share/zope/' ]

from DateTime import DateTime

PREAMBLE_EXTENSIONS_SECTION    = "EXTENSIONSECTION"
PREAMBLE_POLICY_SECTION        = "POLICYOPTIONS"
# FILE & DIRECTORY NAMES
CA_DIR_CERTS                   = "certs"
CA_DIR_CRL                     = "crl"
CA_DIR_PRIVATE                 = "private"
CA_DIR_PRIVATE_OWNCERTS        = os.path.join(CA_DIR_PRIVATE,"keys")
CA_DIR_PRIVATE_OWNCERTS_CONFIG = "%s/keys" % CA_DIR_PRIVATE
CA_DIR_PRIVATE_CERTS           = os.path.join(CA_DIR_PRIVATE,"certs")
CA_FILE_SERIAL                 = "serial"
CA_FILE_CRL                    = "crl.pem"
CA_FILE_CACERT                 = "cacert.pem"
CA_FILE_CAKEY                  = "cakey.pem"
CA_FILE_RAND                   = '.rand'
CA_FILE_INDEX_CERTS            = "index.txt"
#
OPENSSL_CONF_FILE              = "openssl.conf"
CA_EXTTEMPLATES_FILE_DIR       = "ext_templates_dir"
# PROPIETARY FILES
CA_FILE_INDEX_PRIVATE_OWNCERTS = "index.txt"
CA_FILE_INDEX_PRIVATE_CERTS    = "index.txt"
# PROPIETARY FILES STRUCTURE
CA_FILE_INDEX_PRIVATE_OWNCERTS_FIELDS = ['ident','fingerprint','subject','issuer','modulus']
CA_FILE_INDEX_PRIVATE_CERTS_FIELDS = ['ident','fingerprint','subject','issuer','modulus']

# SECTIONS
SECTION_CA                     = 'ca'
SECTION_CACONF                 = 'ca_conf'
SECTION_POLICYMATCH            = 'policy_match'
SECTION_POLICYANY              = 'policy_any'
SECTION_REQ                    = 'req'
SECTION_NEWOIDS                = 'new_oids'
SECTION_REQDN                  = 'req_distinguished_name'
SECTION_REQATTR                = 'req_attributes'
SECTION_ENGINE                 = 'engine'
SECTION_EXTENSIONS             = 'extension_section'
POLICY_SECTION                 = 'POLICY_SECTION'

# DEFAULT SECTION
OPTION_HOME                  = 'HOME'
OPTION_RANDFILE              = 'RANDFILE'
OPTION_OIDFILE               = 'oid_file'
OPTION_OIDSECTION            = 'oid_section'

# CA SECTION OPTIONS
OPTION_CACONF_DIR             = 'dir'
OPTION_CACONF_CERTS           = 'certs'
OPTION_CACONF_CRLDIR          = 'crl_dir'
OPTION_CACONF_DATABASE        = 'database'
OPTION_CACONF_NEWCERTSDIR     = 'new_certs_dir'
OPTION_CACONF_CERTIFICATE     = 'certificate'
OPTION_CACONF_SERIAL          = 'serial'
OPTION_CACONF_CRL             = 'crl'
OPTION_CACONF_PRIVATEKEY      = 'private_key'
OPTION_CACONF_RANDFILE        = 'RANDFILE'
OPTION_CACONF_EXTENSIONS      = 'extensions'
OPTION_CACONF_X509EXTENSIONS  = 'x509_extensions'
OPTION_CACONF_DEFAULTDAYS     = 'default_days'
OPTION_CACONF_DEFAULTCRLDAYS  = 'default_crl_days'
OPTION_CACONF_DEFAULTCRLHOURS = 'default_crl_hours'
OPTION_CACONF_DEFAULTMD       = 'default_md'
OPTION_CACONF_PRESERVE        = 'preserve'
OPTION_CACONF_POLICY          = 'policy'
OPTION_CACONF_SECCLABENABLEDPOLICIES = 'secclab_enabledpolicies'
OPTION_CACONF_SECCLABDISABLEDPOLICIES = 'secclab_disabled_policies'
OPTION_CACONF_REQ             = 'req'

# POLICY SECTION OPTIONS
OPTION_POLICY_COUNTRYNAME           = 'countryName'
OPTION_POLICY_STATEORPROVINCENAME   = 'stateOrProvinceName'
OPTION_POLICY_LOCALITYNAME          = 'localityName'
OPTION_POLICY_ORGANIZATIONNAME      = 'organizationName'
OPTION_POLICY_ORGANIZATIONUNITNAME  = 'organizationalUnitName'
OPTION_POLICY_COMMONNAME            = 'commonName'
OPTION_POLICY_EMAILADDRESS          = 'emailAddress'

# POLICY SECTION VALUES
VALUE_POLICY_MATCH    = 'match'
VALUE_POLICY_OPTIONAL = 'optional'
VALUE_POLICY_SUPPLIED = 'supplied'

# REQ SECTION OPTIONS
OPTION_REQ_DEFAULTBITS       = 'default_bits'
OPTION_REQ_DEFAULTKEYFILE    = 'default_keyfile'
OPTION_REQ_DISTINGUISHEDNAME = 'distinguished_name'
OPTION_REQ_ATTRIBUTES        = 'attributes'
OPTION_REQ_X509EXTENSIONS    = 'x509_extensions'
OPTION_REQ_PROMPT            = 'prompt'
OPTION_REQ_ENCRYPTRSAKEY     = 'encrypt_rsa_key'

# REQ-ATTRIBUTES SECTION OPTIONS
OPTION_REQATTR_CHALLENGEPASSWORD = 'challengePassword'
OPTION_REQATTR_UNSTRUCTUREDNAME  = 'unstructuredName'

# SUFFIXES FOR REQUES DISTINGUISED NAME
SUFFIX_REQDN_DEFAULT = 'default'
SUFFIX_REQDN_MAX     = 'max'
SUFFIX_REQDN_MIN     = 'min'
SUFFIX_REQDN_REGEX   = 'regex'

# CA SECTION OPTIONS
OPTION_CA_DEFAULTCA = 'default_ca'

# NOT EXTENSION POLICY OPTIONS
OPTION_NOEXT_VALIDITY             = 'PRIVATE_VALIDITY'
OPTION_NOEXT_BITSDOWN             = 'PRIVATE_BITDOWN'
OPTION_NOEXT_BITSUP               = 'PRIVATE_BITUP'

# EXTENSION SECTION OPTIONS
OPTION_EXT_BASICCONSTRAINTS       = 'basicConstraints'
OPTION_EXT_KEYUSAGE               = 'keyUsage'
OPTION_EXT_EXTENDEDKEYUSAGE       = 'extendedKeyUsage'
OPTION_EXT_SUBJECTKEYIDENTIFIER   = 'subjectKeyIdentifier'
OPTION_EXT_AUTHORITYKEYIDENTIFIER = 'authorityKeyIdentifier'
OPTION_EXT_SUBJECTKEYIDENTIFIER   = 'subjectKeyIdentifier'
OPTION_EXT_SUBJECTALTNAME         = 'subjectAltName'
OPTION_EXT_ISSUERALTNAME          = 'issuerAltName'
OPTION_EXT_NSCERTTYPE             = 'nsCertType'
#OPTION_EXT_NSBASEURL              = 'nsBaeUrl'
OPTION_EXT_NSCOMMENT              = 'nsComment'
OPTION_EXT_NSCAREVOCATIONURL      = 'nsCaRevocationUrl'
OPTION_EXT_NSRENEWALURL           = 'nsRenewalUrl'
OPTION_EXT_NSCAPOLICYURL          = 'nsCaPolicyUrl'
OPTION_EXT_CRLDISTRIBUTIONPOINTS  = 'crlDistributionPoints'
OPTION_EXT_CERTIFICATEPOLICIES    = 'certificatePolicies'
# EXTENSION SECTION VALUES
VALUE_EXT_CRITICAL = 'critical'
## BASIC CONSTRAINTS OPTION VALUES
VALUE_EXT_BASICCONSTRAINTS_CATRUE    = 'CA:TRUE'
VALUE_EXT_BASICCONSTRAINTS_CAFALSE   = 'CA:FALSE'
VALUE_EXT_BASICCONSTRAINTS_PATHLEN_n = 'pathlen:%s'
## KEY USAGE EXTENSION VALUES
VALUE_EXT_KEYUSAGE_DIGITALSIGNATURE = 'digitalSignature'
VALUE_EXT_KEYUSAGE_NONREPUDIATION   = 'nonRepudiation'
VALUE_EXT_KEYUSAGE_KEYENCIPHERMENT  = 'keyEncipherment'
VALUE_EXT_KEYUSAGE_DATAENCIPHERMENT = 'dataEncipherment'
VALUE_EXT_KEYUSAGE_KEYAGREEMENT     = 'keyAgreement'
VALUE_EXT_KEYUSAGE_KEYCERTSIGN      = 'keyCertSign'
VALUE_EXT_KEYUSAGE_CRLSIGN          = 'cRLSign'
VALUE_EXT_KEYUSAGE_ENCIPHERONLY     = 'encipherOnly'
VALUE_EXT_KEYUSAGE_DECIPHERONLY     = 'decipherOnly'
## EXTENDED KEY USAGE EXTENSION VALUES
VALUE_EXT_EXTENDEDKEYUSAGE_SERVERAUTH      = 'serverAuth'
VALUE_EXT_EXTENDEDKEYUSAGE_CLIENTAUTH      = 'clientAuth'
VALUE_EXT_EXTENDEDKEYUSAGE_CODESIGNING     = 'codeSigning'
VALUE_EXT_EXTENDEDKEYUSAGE_EMAILPROTECTION = 'emailProtection'
VALUE_EXT_EXTENDEDKEYUSAGE_TIMESTAMPING    = 'timeStamping'
VALUE_EXT_EXTENDEDKEYUSAGE_MSCODEIND       = 'msCodeInd'
VALUE_EXT_EXTENDEDKEYUSAGE_MSCODECOM       = 'msCodeCom'
VALUE_EXT_EXTENDEDKEYUSAGE_MSCTLSIGN       = 'msCTLSign'
VALUE_EXT_EXTENDEDKEYUSAGE_MSSGC           = 'msSGC'
VALUE_EXT_EXTENDEDKEYUSAGE_MSEFS           = 'msEFS'
VALUE_EXT_EXTENDEDKEYUSAGE_NSSGC           = 'nsSGC'
## SUBJECT KEY IDENTIFIER EXTENSION VALUES
VALUE_EXT_SUBJECTKEYIDENTIFIER_HASH = 'hash'
## AUTHORIRY KEY IDENTIFIER EXTENSION VALUES
VALUE_EXT_AUTHORITYKEYIDENTIFIER_KEYID        = 'keyid'
VALUE_EXT_AUTHORITYKEYIDENTIFIER_ISSUER       = 'issuer'
VALUE_EXT_AUTHORITYKEYIDENTIFIER_KEYIDALWAYS  = 'keyid:always'
VALUE_EXT_AUTHORITYKEYIDENTIFIER_ISSUERALWAYS = 'issuer:always'
## SUBJECT ALT NAME EXTENSION VALUES
VALUE_EXT_SUBJECTALTNAME_EMAIL                = 'email:'
VALUE_EXT_SUBJECTALTNAME_EMAILCOPY            = 'email:copy'
VALUE_EXT_SUBJECTALTNAME_URI                  = 'URI:'
VALUE_EXT_SUBJECTALTNAME_DNS                  = 'DNS:'
VALUE_EXT_SUBJECTALTNAME_RID                  = 'RID:'
VALUE_EXT_SUBJECTALTNAME_IP                   = 'IP:'
## ISSUER ALT NAME EXTENSION VALUES
VALUE_EXT_ISSUERALTNAME_ISSUERCOPY           = 'issuer:copy'
VALUE_EXT_ISSUERALTNAME_URI                  = 'URI:'
VALUE_EXT_ISSUERALTNAME_DNS                  = 'DNS:'
VALUE_EXT_ISSUERALTNAME_RID                  = 'RID:'
VALUE_EXT_ISSUERALTNAME_IP                   = 'IP:'
## NS CERT TYPE EXTENSION VALUES
VALUE_EXT_NSCERTYPE_CLIENT                   = 'client'
VALUE_EXT_NSCERTYPE_SERVER                   = 'server'
VALUE_EXT_NSCERTYPE_EMAIL                    = 'email'
VALUE_EXT_NSCERTYPE_OBJSIGN                  = 'objsign'
VALUE_EXT_NSCERTYPE_RESERVED                 = 'reserved'
VALUE_EXT_NSCERTYPE_SSLCA                    = 'sslCA'
VALUE_EXT_NSCERTYPE_EMAILCA                  = 'emailCA'
VALUE_EXT_NSCERTYPE_OBJCA                    = 'objCA'
## CRL DISTRIBUTION POINTS EXTENSION VALUES
VALUE_EXT_CRLDISTRIBUTIONPOINTS_EMAIL        = 'email:'
VALUE_EXT_CRLDISTRIBUTIONPOINTS_EMAILCOPY    = 'email:copy'
VALUE_EXT_CRLDISTRIBUTIONPOINTS_URI          = 'URI:'
VALUE_EXT_CRLDISTRIBUTIONPOINTS_DNS          = 'DNS:'
VALUE_EXT_CRLDISTRIBUTIONPOINTS_RID          = 'RID:'
VALUE_EXT_CRLDISTRIBUTIONPOINTS_IP           = 'IP:'
## CERTIFICATE POLICIES EXTENSION VALUES
VALUE_EXT_CERTIFICATEPOLICIES_IA5ORG           = 'ia5org'
VALUE_EXT_CERTIFICATEPOLICIES_POLICYIDENTIFIER = 'policyIdentifier'
VALUE_EXT_CERTIFICATEPOLICIES_CPS_N            = 'CPS.%d'
VALUE_EXT_CERTIFICATEPOLICIES_USERNOTICE_N     = 'userNotice.%d'
## USER NOTICE OPTIONS
OPTION_EXT_USERNOTICE_EXPLICITTEXT             = 'explicitText'
OPTION_EXT_USERNOTICE_ORGANIZATION             = 'organization'
OPTION_EXT_USERNOTICE_NOTICENUMBRES            = 'noticeNumbers'

#class OpensslConfFile(ConfigParser):
#    """ OpensslConfFile class """
#    class File:
#        def __init__(self,filename,mode='r'):
#            self.filename = filename
#            self.file     = open(filename,mode)
#            self.firstline = 0
#            
#        def readline(self):
#            if not self.firstline:
#                self.firstline = 1
#                return "[DEFAULT]"
#            return self.file.readline()
#    
#        def write(self,s):
#            if s[0:9] != '[DEFAULT]':
#                self.file.write(s)
#        def __del__(self):
#            self.file.close()
#    def __init__(self):
#        ConfigParser.__init__(self)
#
#    def add_section(self,section):
#        if section == 'DEFAULT':
#            raise DuplicateSectionError(section)
#        ConfigParser.add_section(self,section)
#    def readfp(self,filename):
#        ConfigParser.readfp(self,self.File(filename,'r'))
#    def read(self,files):
#        raise "ERROR","ERROR NOT IMPLEMENTED"
#    def write(self,filename):
#        ConfigParser.write(self,self.File(filename,'w'))
#    def optionxform(self, optionstr):
#        return optionstr

CONFIG_FIRST_LINE           = "### DON'T EDIT THIS FILE. IT IS AUTOMATICALY GENERATED"
CONFIG_INIT_VARIABLE_LIST   = "# INIT VARIABLE LIST"
CONFIG_FINISH_VARIABLE_LIST = "# FINISH VARIABLE LIST"

class OpensslConfFile(ConfigParser):
    class File:
        def __init__(self,filename,mode='r'):
            if mode == 'w':
                if os.path.exists(filename):
                    shutil.copyfile(filename,"%s.old"%filename)
            self.filename = filename
            self.file     = open(filename,mode)
            self.give_next_default_line = 0
            
        def readline(self):
            if self.give_next_default_line == 1:
                self.give_next_default_line = 0
                return "[DEFAULT]"
            return self.file.readline()
    
        def write(self,s):
            if s[0:9] != '[DEFAULT]':
                self.file.write(s)
        def __del__(self):
            self.file.close()

    def __init__(self):
        ConfigParser.__init__(self)
        self.variable_list = {}

    def _read_variable_hack(self,fp):
        line = fp.readline()
        line = string.replace(line,'\r','').replace('\n','')
        if line != CONFIG_FIRST_LINE:
            raise "CONFIG_ERROR","NOT VALID CONFIG FILE (%s)" % line
        line = fp.readline()
        line = string.replace(line,'\r','').replace('\n','')
        if line != CONFIG_INIT_VARIABLE_LIST:
            raise "CONFIG_ERROR","NOT VALID CONFIG FILE (%s)" % line
        line = fp.readline()
        while line:
            line = string.replace(line,'\r','').replace('\n','')
            if line == CONFIG_FINISH_VARIABLE_LIST:
                break
            var,val = string.strip(line).split('=',2)
            var = var.strip()
            val = val.strip()
            self.variable_list[var] = val
            line = fp.readline()
        
    def _write_variable_hack(self,fp):
        fp.write(CONFIG_FIRST_LINE)
        fp.write("\n");
        fp.write(CONFIG_INIT_VARIABLE_LIST)
        fp.write("\n");
        for (var,val) in self.variable_list.items():
            fp.write("%s = %s\n" % (var,val))
        fp.write(CONFIG_FINISH_VARIABLE_LIST)
        fp.write("\n");
        
    def add_variable(self,var,val):
        self.variable_list[var] = val
        
    def add_section(self,section):
        if section == 'DEFAULT':
            raise DuplicateSectionError(section)
        ConfigParser.add_section(self,section)

    def readfp(self,filename):
        file_fp = self.File(filename,'r')
        self._read_variable_hack(file_fp)
        file_fp.give_next_default_line = 1
        ConfigParser.readfp(self,file_fp)

    def read(self,files):
        raise "ERROR","ERROR NOT IMPLEMENTED"

    def write(self,filename):
        file_fp = self.File(filename,'w')
        self._write_variable_hack(file_fp)
        ConfigParser.write(self,file_fp)

    def optionxform(self, optionstr):
        return optionstr



    

def DaysFromNow(toDate):
    """ Calculate days from now """
    date1 = DateTime()
    date2 = DateTime(toDate)
    
    return str(date2 - date1).split('.')[0]

def ExecuteOpensslCommand(command,input):
    """ Execute a command """

    aux = string.split(input,"\n")
    inp = []
    for i in aux:
        inp.append("%s\n" % i)
    inp.append(aux[-1])
        
    output = ""
    error = ""

    if sys.platform[:3] == "win":
        if len(input) == 0:
            child = process.Popen(command,stdout=process.PIPE,stderr=process.PIPE)
            output,error = child.communicate()
        else:
            child = process.Popen(command,stdin=process.PIPE,stdout=process.PIPE,stderr=process.PIPE)
            output,error = child.communicate(''.join(inp))
        return child.returncode,output,error
    else:
    
        child = popen2.Popen3(command,1)

        fcntl.fcntl(child.tochild.fileno(),FCNTL.F_SETFL,os.O_NONBLOCK)
        fcntl.fcntl(child.fromchild.fileno(),FCNTL.F_SETFL,os.O_NONBLOCK)
        fcntl.fcntl(child.childerr.fileno(),FCNTL.F_SETFL,os.O_NONBLOCK)

        to_w = [child.tochild]
        while 1:

            (r,w,e) = select.select([child.fromchild],to_w,[child.childerr],1)

            if len(r) > 0:
                output = output + child.fromchild.read()
            if len(w) > 0:
                if inp is not None:
                    child.tochild.write(inp[0]);
                    if len(inp) == 1:
                        inp = None
                    else:
                        inp = inp[1:]
                else:
                    child.tochild.close()
                    to_w = []
            if len(e) > 0:
                error = error + child.childerr.read()

            if child.poll() != -1:
                break            

        output = output + child.fromchild.read()
        error = error + child.childerr.read()
        child.fromchild.close()
        child.childerr.close()
        if to_w != []:
            try:
                child.tochild.close()
            except:
                pass
        # rc = os.system(command)
        rc = child.poll()

    return rc,output,error

def OpensslGetInfoCert(file):
    command = OPENSSL_BIN + ' x509 -in "%s" -noout -fingerprint -issuer -subject -hash -modulus -purpose -startdate -enddate' % file
    rc,output,error = ExecuteOpensslCommand(command,"")
    if rc:
        raise "OpensslError",error

    values = string.split(output,"\n")
    return {'fingerprint':values[0].split('=')[-1].replace(':',''),
            'issuer'     :string.join(values[1].split('=')[1:],'=').strip(),
            'subject'    :string.join(values[2].split('=')[1:],'=').strip(),
            'ident'      :values[3].split('=')[-1].strip(),
            'modulus'    :values[4].split('=')[-1].strip(),
            'purpose'    :values[5].split('=')[-1],
            'startdate'  :values[6].split('=')[-1].strip(),
            'enddate'    :values[7].split('=')[-1].strip()
            }
def OpensslgetInfoInlineCert(certificate,pem):
    if pem == 1:
        inform = 'pem'
    else:
        inform = 'der'
    command = OPENSSL_BIN + ' x509 -inform '+inform+' -noout -fingerprint -issuer -subject -hash -modulus -purpose'
    rc,output,error = ExecuteOpensslCommand(command,certificate)
    if rc != 0:
        raise "OpensslError",error
    values = string.split(output,"\n")
    return {'fingerprint':values[0].split('=')[-1].replace(':',''),
            'issuer':string.join(values[1].split('=')[1:],'=').strip(),
            'subject':string.join(values[2].split('=')[1:],'=').strip(),
            'ident':values[3].split('=')[-1].strip(),
            'modulus':values[4].split('=')[-1].strip(),
            'purpose':values[5].split('=')[-1]
            }

def OpensslReadConfFile(dir):
    conf = OpensslConfFile()
    conf.readfp(os.path.join(dir,OPENSSL_CONF_FILE))
    return conf

def OpensslWriteConfFile(dir,conf):
    conf.write(os.path.join(dir,OPENSSL_CONF_FILE))


def EnabledPolicyList(conf):
    l = conf.get(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES)
    l = string.strip(l).split(',')
    if len(l) == 1 and string.strip(l[0]) == '':
        return []
    polList = []
    for i in l:
        polList.append(string.strip(i))
        
    return polList

def DisabledPolicyList(conf):
    l = conf.get(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES)
    l = string.strip(l).split(',')
    if len(l) == 1 and string.strip(l[0]) == '':
        return []
    polList = []
    for i in l:
        polList.append(string.strip(i))
        
    return polList

def GetTemplateList(dir):
    templates = []
    for t in os.listdir(os.path.join(dir,CA_EXTTEMPLATES_FILE_DIR)):
        tt = string.split(t,".")
        if len(tt) > 1 and tt[-1] == 'osslcnf':
            templates.append(string.join(tt[:-1],'.'))
    return templates
    
def GetPolicyList(dir):
    def SortPolicyList(e1,e2):
        if (e1['name'] > e2['name']):
            return 1
        elif (e1['name'] < e2['name']):
            return -1
        else:
            return 0
        
    conf = OpensslReadConfFile(dir)
    new_list = []
    l = EnabledPolicyList(conf)
    for i in l:
        new_list.append({'name':i,'enabled':1})
    l = DisabledPolicyList(conf)
    for i in l:
        new_list.append({'name':i,'enabled':0})

    new_list.sort(SortPolicyList)
    return new_list

def UpdatePolicy(dir,old_policy,new_policy,dict):
    old_policy = string.strip(old_policy)
    new_policy = string.strip(new_policy)
    conf = OpensslReadConfFile(dir)
    pol_enabled = EnabledPolicyList(conf)
    pol_disabled = DisabledPolicyList(conf)

    if old_policy != new_policy:
        if new_policy in pol_enabled+pol_disabled:
            raise "OpensslError","New policy name already exists"
        if old_policy in pol_enabled:
            pol_enabled.remove(old_policy)
            pol_enabled.append(new_policy)
            conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES,string.join(pol_enabled,','))
        elif old_policy in pol_disabled:
            pol_disabled.remove(old_policy)
            pol_disabled.append(new_policy)
            conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,string.join(pol_disabled,','))
        else:
            raise "OpensslError","Error on old policy name"

        conf.remove_section("%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,old_policy))
        conf.remove_section("%s_%s" % (PREAMBLE_POLICY_SECTION,old_policy))
        conf.add_section("%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,new_policy))
        conf.add_section("%s_%s" % (PREAMBLE_POLICY_SECTION,new_policy))
        
    Dict2Policy(dict,conf,"%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,new_policy),"%s_%s" % (PREAMBLE_POLICY_SECTION,new_policy))
    OpensslWriteConfFile(dir,conf)

def AddPolicy(dir,name,template):
    conf = OpensslReadConfFile(dir)
    pol_enabled  = EnabledPolicyList(conf)
    pol_disabled = DisabledPolicyList(conf)
    temp_list = GetTemplateList(dir)
    if name in pol_enabled+pol_disabled:
        raise "OpensslError","Policy name already exists"
    if name in temp_list:
        raise "OpensslError","Policy name can not be a template name"

    pol_disabled.append(name)

    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,string.join(pol_disabled,','))
    if template == '':
        dict = NewEmptyPolicy()
    elif template in pol_enabled+pol_disabled:
        dict = Policy2Dict(name ,conf,"%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,template),"%s_%s" % (PREAMBLE_POLICY_SECTION,template))
    elif template in temp_list:
        conf_aux = OpensslConfFile()
        conf_aux.readfp(os.path.join(dir,CA_EXTTEMPLATES_FILE_DIR,"%s.osslcnf" % template))
        dict = Policy2Dict(name ,conf_aux,SECTION_EXTENSIONS,POLICY_SECTION)
    else:
        raise "OpensslError","Template name not foune"
    
    conf.add_section("%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,name))
    conf.add_section("%s_%s" % (PREAMBLE_POLICY_SECTION,name))
    Dict2Policy(dict,conf,"%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,name),"%s_%s" % (PREAMBLE_POLICY_SECTION,name))
    
    OpensslWriteConfFile(dir,conf)
    
def EraseEoln(reg):
    last_value = reg[len(reg)-1]
    if len(last_value) >= 2:
        if last_value[len(last_value)-2:len(last_value)] == '\r\n':
            last_value = last_value[0:len(last_value)-2]
        elif last_value[len(last_value)-1] == '\n':
            last_value = last_value[0:len(last_value)-1]
    elif last_value[len(last_value)-1] == '\n':
        last_value = last_value[0:len(last_value)-1]

    reg[len(reg)-1] = last_value
    return reg

def RemoveCertFromIndexFile(file,ident):
    f = open(file,"r")
    faux = open(file+".new","w")
    line = f.readline()
    while len(line) != 0:
        reg = string.split(line,'|')
        reg = EraseEoln(reg)
        if reg[0] != ident:
            faux.write(line)
        line = f.readline()
    f.close()
    faux.close()
    os.rename(file+".new",file)

def ReadIndexFile(file,field_list):
    f = open(file,"r")
    line = f.readline()
    value_list = []
    r = range(len(field_list))
    while (len(line) != 0):
        reg = string.split(line,'|')
        reg = EraseEoln(reg)
        if len(field_list) != len(reg):
            raise "OpensslError","Error in file format '%s'" % file
        elem = {}
        for i in r:
            elem[field_list[i]] = reg[i]
        value_list.append(elem)
        line = f.readline()
    f.close()
    return value_list
    
def GetCACertList(dir,password):
    def sort_function(e1,e2):
        if e1['issuer'] > e2['issuer']:
            return 1
        elif e1['issuer'] < e2['issuer']:
            return -1
        else:
            if e1['subject'] == e1['issuer'] and e2['subject'] == e2['issuer']:
                return 0
            elif e1['subject'] == e1['issuer']:
                return 1
            else:
                return 0
    
    l1 = ReadIndexFile(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_INDEX_PRIVATE_OWNCERTS),
                       CA_FILE_INDEX_PRIVATE_OWNCERTS_FIELDS)
    l2 = ReadIndexFile(os.path.join(dir,CA_DIR_PRIVATE_CERTS,CA_FILE_INDEX_PRIVATE_CERTS),
                       CA_FILE_INDEX_PRIVATE_CERTS_FIELDS)

    
    own_modulus = []
    own_modulus.append(GetCAKeyModulus(dir,password))
    l = []
    for elem in l1:
        # own_modulus.append(elem['modulus'])
        if elem['modulus'] in own_modulus:
            elem['own'] = 1
        else:
            elem['own'] = 0
        l.append(elem)
    for elem in l2:
        if elem['modulus'] in own_modulus:
            elem['own'] = 1
        else:
            elem['own'] = 0
        l.append(elem)
    l.sort(sort_function)
    return l

def VerifyCACert(dir,cert_id):
    return 0

def GetCACert(dir,password,ident):
    cert_list = GetCACertList(dir,password)
    for cert in cert_list:
        if cert['ident'] == ident:
            if (cert['own']):
                filename = os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT)
            else:
                filename = os.path.join(dir,CA_DIR_PRIVATE_CERTS,"%s.0" % ident)
            f = open(filename,"r")
            certificate_pem = f.read()
            f.close()
            return certificate_pem
    return None

def RemoveCACert(dir,password,ident):
    RemoveCertFromIndexFile(os.path.join(dir,CA_DIR_PRIVATE_CERTS,CA_FILE_INDEX_PRIVATE_CERTS),ident)
    return "Certificate removed"

def ImportCACert(dir,certificate,password,pem,path):
    info = OpensslgetInfoInlineCert(certificate,pem)
    cert_list = GetCACertList(dir,password)
    for cert in cert_list:
        if (info['ident'] == cert['ident']) and (not cert['own']):
            return "There is a copy of this certificate in the list (or a certificate with the same subject)"
    AddCACertificate(dir,certificate,info,pem)
    return "Certificate imported"

def NewCA(dir,passwd,form):
    os.mkdir (os.path.join(dir,CA_DIR_CERTS))
    os.mkdir (os.path.join(dir,CA_DIR_CRL))
    os.mkdir (os.path.join(dir,CA_DIR_PRIVATE))
    os.mkdir (os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS))
    os.mkdir (os.path.join(dir,CA_DIR_PRIVATE_CERTS))
    os.mkdir (os.path.join(dir,CA_EXTTEMPLATES_FILE_DIR))
    f = open(os.path.join(dir,CA_FILE_INDEX_CERTS),'w');f.close()
    f = open(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_INDEX_PRIVATE_OWNCERTS),'w');f.close()
    f = open(os.path.join(dir,CA_DIR_PRIVATE_CERTS,CA_FILE_INDEX_PRIVATE_CERTS),'w');f.close()
    f = open(os.path.join(dir,CA_FILE_SERIAL),'w');f.write('01');f.close()
    
    conf = OpensslConfFile()
    
    defaults = conf.defaults()
    
    defaults['HOME']        = '.'
    ##defaults['RANDFILE']    = '$ENV::HOME/.rnd'
    defaults['RANDFILE']    = "$dir/.rnd"
    # defaults['oid_file']    = '$ENV::HOME/.oid'
    defaults['oid_section'] = 'new_oids'
    
    # SECTION_NEWOIDS
    conf.add_section(SECTION_NEWOIDS)
    
    # SECTION_CA
    conf.add_section(SECTION_CA)
    conf.set(SECTION_CA,OPTION_CA_DEFAULTCA,SECTION_CACONF)
    
    # SECTION_CACONF
    conf.add_section(SECTION_CACONF)
    if sys.platform[:3] == "win":
        conf.add_variable(OPTION_CACONF_DIR,'%s' % dir.replace("\\","/"))
    else:
        conf.add_variable(OPTION_CACONF_DIR,'%s' % dir)
    #conf.set(SECTION_CACONF,OPTION_CACONF_DIR,'%s' % dir)
    # conf.set(SECTION_CACONF,OPTION_CACONF_CERTS,'$dir/%s' % CA_DIR_CERTS)
    conf.set(SECTION_CACONF,OPTION_CACONF_NEWCERTSDIR,"$dir/%s" % CA_DIR_CERTS)
    conf.set(SECTION_CACONF,OPTION_CACONF_CRLDIR,"$dir/%s" % CA_DIR_CRL)
    conf.set(SECTION_CACONF,OPTION_CACONF_DATABASE,"$dir/%s" % CA_FILE_INDEX_CERTS)
    conf.set(SECTION_CACONF,OPTION_CACONF_CERTIFICATE,"$dir/%s/%s" % (CA_DIR_PRIVATE_OWNCERTS_CONFIG,CA_FILE_CACERT))
    conf.set(SECTION_CACONF,OPTION_CACONF_SERIAL,"$dir/%s" % CA_FILE_SERIAL)
    conf.set(SECTION_CACONF,OPTION_CACONF_CRL,"$dir/%s" % CA_FILE_CRL)
    conf.set(SECTION_CACONF,OPTION_CACONF_PRIVATEKEY,"$dir/%s/%s" % (CA_DIR_PRIVATE_OWNCERTS_CONFIG,CA_FILE_CAKEY))
    conf.set(SECTION_CACONF,OPTION_CACONF_RANDFILE,"$dir/%s/%s" % (CA_DIR_PRIVATE,CA_FILE_RAND))
    # conf.set(SECTION_CACONF,OPTION_CACONF_X509EXTENSIONS,'')
    conf.set(SECTION_CACONF,OPTION_CACONF_DEFAULTDAYS,'365')
    conf.set(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLDAYS,'30')
    conf.set(SECTION_CACONF,OPTION_CACONF_DEFAULTMD,'sha1')
    conf.set(SECTION_CACONF,OPTION_CACONF_PRESERVE,'no')
    conf.set(SECTION_CACONF,OPTION_CACONF_POLICY,SECTION_POLICYMATCH)
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES,'')
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,'')
    conf.set(SECTION_CACONF,OPTION_CACONF_REQ,SECTION_REQ)
    # SECTION_POLICYMATCH
    conf.add_section(SECTION_POLICYMATCH)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_COUNTRYNAME,VALUE_POLICY_MATCH)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_STATEORPROVINCENAME,VALUE_POLICY_MATCH)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_ORGANIZATIONNAME,VALUE_POLICY_MATCH)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_ORGANIZATIONUNITNAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_COMMONNAME,VALUE_POLICY_SUPPLIED)
    conf.set(SECTION_POLICYMATCH,OPTION_POLICY_EMAILADDRESS,VALUE_POLICY_OPTIONAL)
    # SECTION_POLICYANY
    conf.add_section(SECTION_POLICYANY)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_COUNTRYNAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_STATEORPROVINCENAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_LOCALITYNAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_ORGANIZATIONNAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_ORGANIZATIONUNITNAME,VALUE_POLICY_OPTIONAL)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_COMMONNAME,VALUE_POLICY_SUPPLIED)
    conf.set(SECTION_POLICYANY,OPTION_POLICY_EMAILADDRESS,VALUE_POLICY_OPTIONAL)
    # SECTION_REQ
    conf.add_section(SECTION_REQ)
    conf.set(SECTION_REQ,OPTION_REQ_DEFAULTBITS,'1024')
    conf.set(SECTION_REQ,OPTION_REQ_DEFAULTKEYFILE,'privkey.pem')
    conf.set(SECTION_REQ,OPTION_REQ_DISTINGUISHEDNAME,SECTION_REQDN)
    conf.set(SECTION_REQ,OPTION_REQ_ATTRIBUTES,SECTION_REQATTR)
    # conf.set(SECTION_REQ,OPTION_REQ_X509EXTENSIONS,'')
    # SECTION_REQDN
    ##conf.add_section(SECTION_REQDN)
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COUNTRYNAME,'Country Name (2 letter code)')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COUNTRYNAME+'_'+SUFFIX_REQDN_DEFAULT,'ES')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COUNTRYNAME+'_'+SUFFIX_REQDN_MIN,'2')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COUNTRYNAME+'_'+SUFFIX_REQDN_MAX,'2')
    ##
    ##conf.set(SECTION_REQDN,OPTION_POLICY_STATEORPROVINCENAME,
    ##         'State or Province Name (full name)')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_STATEORPROVINCENAME+'_'+SUFFIX_REQDN_DEFAULT,
    ##         'Madrid')
    ### conf.set(SECTION_REQDN,OPTION_POLICY_LOCALITYNAME,'Locality Name (eg, city)')
    ### conf.set(SECTION_REQDN,OPTION_POLICY_LOCALITYNAME+'_'+SUFFIX_REQDN_DEFAULT,'Madrid')
    ##
    ##conf.set(SECTION_REQDN,OPTION_POLICY_ORGANIZATIONNAME,
    ##         'Organization Name (eg, company)')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_ORGANIZATIONNAME+'_'+SUFFIX_REQDN_DEFAULT,
    ##         'My Company Ltd')
    ##
    ##conf.set(SECTION_REQDN,OPTION_POLICY_ORGANIZATIONUNITNAME,
    ##         'Organizational Unit Name (eg, section)')
    ##
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COMMONNAME,
    ##         'Common Name (eg, your name or your server\'s hostname)')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_COMMONNAME+'_'+SUFFIX_REQDN_MAX,'64')
    ##
    ##conf.set(SECTION_REQDN,OPTION_POLICY_EMAILADDRESS,'Email Address')
    ##conf.set(SECTION_REQDN,OPTION_POLICY_EMAILADDRESS+'_'+SUFFIX_REQDN_MAX,'40')
    ##
    ### SECTION_REQATTR
    ##conf.add_section(SECTION_REQATTR)
    ##conf.set(SECTION_REQATTR,OPTION_REQATTR_CHALLENGEPASSWORD,'A challenge password')
    ##conf.set(SECTION_REQATTR,OPTION_REQATTR_CHALLENGEPASSWORD+'_'+SUFFIX_REQDN_MIN,'4')
    ##conf.set(SECTION_REQATTR,OPTION_REQATTR_CHALLENGEPASSWORD+'_'+SUFFIX_REQDN_MAX,'20')
    ##conf.set(SECTION_REQATTR,OPTION_REQATTR_UNSTRUCTUREDNAME,'An optional company name')
    ### SECTION_ENGINE
    ##conf.add_section(SECTION_ENGINE)
    ##conf.set(SECTION_ENGINE,'default','openssl')
    
    conf.write(os.path.join(dir,OPENSSL_CONF_FILE))
    
    # CA REQ FILE
    confReq = OpensslConfFile()
    
    # SECTION_REQ PARA LA CA
    confReq.add_section(SECTION_REQ)
    confReq.set(SECTION_REQ,OPTION_REQ_DEFAULTBITS,form['bits'])
    confReq.set(SECTION_REQ,OPTION_REQ_DEFAULTKEYFILE,'privkey.pem')
    confReq.set(SECTION_REQ,OPTION_REQ_DISTINGUISHEDNAME,SECTION_REQDN)
    confReq.set(SECTION_REQ,OPTION_REQ_PROMPT,'no')
    confReq.set(SECTION_REQ,OPTION_REQ_X509EXTENSIONS,'v3_ca')
    # SECTION_V3_CA
    confReq.add_section('v3_ca')
    confReq.set('v3_ca',OPTION_EXT_SUBJECTKEYIDENTIFIER,'hash')
    # confReq.set('v3_ca',OPTION_EXT_AUTHORITYKEYIDENTIFIER,'keyid:always,issuer:always')
    confReq.set('v3_ca',OPTION_EXT_BASICCONSTRAINTS,'CA:true,pathlen:%s' % form['pathlen'])
    confReq.set('v3_ca',OPTION_EXT_KEYUSAGE,'cRLSign,keyCertSign')
    # SECTION_REQDN
    confReq.add_section(SECTION_REQDN)
    confReq.set(SECTION_REQDN,OPTION_POLICY_COUNTRYNAME,form['countryName'])
    confReq.set(SECTION_REQDN,OPTION_POLICY_STATEORPROVINCENAME,
                form['stateOrProvinceName'])
    confReq.set(SECTION_REQDN,OPTION_POLICY_ORGANIZATIONNAME,
                form['organizationName'])
    confReq.set(SECTION_REQDN,OPTION_POLICY_ORGANIZATIONUNITNAME,
                form['organizationUnitName'])
    confReq.set(SECTION_REQDN,OPTION_POLICY_COMMONNAME,
                form['commonName'])
    confReq.set(SECTION_REQDN,OPTION_POLICY_EMAILADDRESS,form['emailAddress'])
    confReq.write(os.path.join(dir,"careq.conf"))
    
    days = DaysFromNow(form['to'])
    
    command = OPENSSL_BIN + ' req -new -x509 -passout stdin -keyout "%s" -out "%s" -days %s -config "%s"' % (os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY),os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT),days,os.path.join(dir,"careq.conf"))


    rc,output,error = ExecuteOpensslCommand(command,"%s" % passwd)
    if rc:
        raise "OpensslError",error
        
    os.chmod(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY),0400)
    f = open(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_INDEX_PRIVATE_OWNCERTS),'a')
    info = OpensslGetInfoCert(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT))
    for i in CA_FILE_INDEX_PRIVATE_OWNCERTS_FIELDS[:-1]:
        f.write("%s|" % info[i])
    f.write("%s\n" % info[CA_FILE_INDEX_PRIVATE_OWNCERTS_FIELDS[-1]])
    f.close()

def AddCACertificate(dir,certificate,certinfo,pem):
    if pem == 1:
        inform = 'pem'
    else:
        inform = 'der'
    command = OPENSSL_BIN + ' x509 -inform %s -outform pem -out "%s"' % (inform,os.path.join(dir,CA_DIR_PRIVATE_CERTS,"%s.0" % certinfo['ident']))
    rc,output,error = ExecuteOpensslCommand(command,certificate)
    if rc:
        raise "OpensslError",error
    f = open(os.path.join(dir,CA_DIR_PRIVATE_CERTS,CA_FILE_INDEX_PRIVATE_CERTS),'a')
    for i in CA_FILE_INDEX_PRIVATE_CERTS_FIELDS[:-1]:
        f.write("%s|" % certinfo[i])
    f.write("%s\n" % certinfo[CA_FILE_INDEX_PRIVATE_CERTS_FIELDS[-1]])
    f.close()


def GetCAKeyModulus(dir,passwd):
    command = OPENSSL_BIN + ' rsa -passin stdin -in "%s" -noout -modulus' % os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY)
    input = "%s\n" % passwd
    rc,output,error = ExecuteOpensslCommand(command,input)
    return string.split(output,"=")[-1].strip()


def OpensslGetCertificateInfo(dir,password,own,ident):
    cert_list = GetCACertList(dir,password)
    for cert in cert_list:
        if (cert['ident'] == ident) and (cert['own'] == own):
            if (cert['own']):
                command = OPENSSL_BIN + ' x509 -in "%s" -text -noout' % os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT)
            else:
                command = OPENSSL_BIN + ' x509 -in "%s" -text -noout' % os.path.join(dir,CA_DIR_PRIVATE_CERTS,"%s.0" % cert['ident'])
            rc,output,error = ExecuteOpensslCommand(command,'')
            if rc:
                raise "OpensslError",error

            return output
        
    raise "OpensslError","Certificate does not exist"


def WriteExtensionFromDict(dict,extension,conf,section,option):
    if not conf.has_section(section):
        conf.add_section(section)

    if not dict[extension].has_key('extension'):
        if dict[extension]['type']['type'] == 'number':
            conf.set(section,option,"%d" % dict[extension]['value'])
        elif dict[extension]['type']['type'] == 'string':
            conf.set(section,option,"%s" % dict[extension]['value'])
        else:
            raise "OpensslError","Invalid type for option %s %s" % (extension,dict[extension]['type']['type'])
    else:

        if dict[extension]['extension'] == 'notpresent':
            return
        # STRING  
        if dict[extension]['type']['type'] == 'string':
            # LOOK FOR RESERVED WORD
            if dict[extension]['type'].has_key('reserved_words'):
                if dict[extension]['value'] in dict[extension]['type']['reserved_words']:
                    value = '%s' % dict[extension]['value']
                else:
                    value = '"%s"' % dict[extension]['value']
            else:
                value = '"%s"' % dict[extension]['value']
            # SET VALUE AND CRITICAL OPTION    
            if dict[extension]['extension'] == 'present':
                conf.set(section,option,'%s' % value)
            elif dict[extension]['extension'] == 'critical':
                conf.set(section,option,'critical,%s' % value)
        # DICT
        elif dict[extension]['type']['type'] == 'dict':
            values = []
            ext_val = dict[extension]['value']
            for op in ext_val.keys():
                if type(ext_val[op]) == types.DictType:
                    if ext_val[op]['present'] == 'true':
                        if ext_val[op]['type']['type'] == 'string':
                            # LOOK FOR RESERVED WORD
                            if ext_val[op]['type'].has_key('reserved_words'):
                                if ext_val[op]['value'] in ext_val[op]['type']['reserved_words']:
                                    value = '%s' % ext_val[op]['value']
                                else:
                                    value = '"%s"' % ext_val[op]['value']
                            else:
                                value = '"%s"' % ext_val[op]['value']
                            # SET VALUE
                            values.append('%s:%s' % (op,value))
                        else:
                            if ext_val[op]['value'] is None:
                                values.append('%s' % op)
                            else:
                                values.append('%s:%s' % (op,ext_val[op]['value']))
                else:
                    if (ext_val[op] == 'true'):
                        values.append('%s' % op)
            if dict[extension]['extension'] == 'present':
                conf.set(section,option,'%s' % string.join(values,','))
            elif dict[extension]['extension'] == 'critical':
                if (len(values) != 0):
                    conf.set(section,option,'critical,%s' % string.join(values,','))
                else:
                    conf.set(section,option,'critical')
        elif dict[extension]['type']['type'] == 'choice':
            values = []
            ext_val = dict[extension]['value']
            for op in ext_val.keys():
                if (ext_val[op] == 'true'):
                    values.append('%s' % op)
            if len(values) > 1:
                raise "OpensslError","Invalid value from choice type %s" % extension
            if dict[extension]['extension'] == 'present':
                if len(values) == 1:
                    conf.set(section,option,'%s' % values[0])
                else:
                    conf.set(section,option,'')
            elif dict[extension]['extension'] == 'critical':
                if len(values) == 1:
                    conf.set(section,option,'critical,%s' % values[0])
                else:
                    conf.set(section,option,'critical')

        else:
                raise "OpensslError","Invalid type for extension %s %s" % (extension,dict[extension]['type']['type'])

def FillDictByExtension(dict,extension,conf,section,option):

    if not dict[extension].has_key('extension'):
        value = conf.get(section,option)
        if dict[extension]['type']['type'] == 'string':
            dict[extension]['value'] = value
        elif dict[extension]['type']['type'] == 'number':
            dict[extension]['value'] = int(value)
        else:
            raise "OpensslError","Invalid policy configuration (in option %s %s)" % (extension,op)
    else:
        if conf.has_option(section,option):
            values = string.strip(conf.get(section,option))
            if values == '':
                dict[extension]['extension'] = 'present'
                return
            values = values.split(',')
            if values[0] == 'critical':
                dict[extension]['extension'] = 'critical'
                values = values[1:]
            else:
                dict[extension]['extension'] = 'present'
            for i in values:
                reg = string.split(i,':')
                op = string.strip(reg[0])
                if len(reg) == 1:
                    val = None
                else:
                    val = reg[1]
                
                if dict[extension]['type']['type'] == 'dict':
                    if not dict[extension]['value'].has_key(op):
                        raise "OpensslError","Invalid policy configuration (in extension %s %s)" % (extension,op)
                    if type(dict[extension]['value'][op]) == types.DictType:
                        if dict[extension]['value'][op]['type']['type']  == 'string':
                            if val[0]=='"' and val[-1]=='"':
                                val = val[1:-1]
                        dict[extension]['value'][op]['present'] = 'true'
                        dict[extension]['value'][op]['value'] = val
                    else:
                        dict[extension]['value'][op] = 'true'
                elif dict[extension]['type']['type'] == 'string':
                    if op[0]=='"' and op[-1]=='"':
                        op = op[1:-1]
                    dict[extension]['value'] = op
                elif dict[extension]['type']['type'] == 'choice':
                    dict[extension]['value'][op] = 'true'

                

                
def NewEmptyPolicy():
    return {'name':'',
            OPTION_NOEXT_VALIDITY:{'value':0,
                                   'type':{'type':'number'}},
            OPTION_NOEXT_BITSDOWN:{'value':0,
                                   'type':{'type':'number'}},
            OPTION_NOEXT_BITSUP:  {'value':0,
                                   'type':{'type':'number'}},
            OPTION_EXT_BASICCONSTRAINTS:{'value':{'CA':{'value':'false','present':'true','type':{'type':'boolean'}},
                                                  'pathlen':{'value':'0','present':'false','type':{'type':'number'}}},
                                         'type':{'type':'dict'},
                                         'extension':'notpresent'},
            OPTION_EXT_KEYUSAGE:{'value':{'digitalSignature' :'false',
                                          'nonRepudiation'   :'false',
                                          'keyEncipherment'  :'false',
                                          'dataEncipherment' :'false',
                                          'keyAgreement'     :'false',
                                          'keyCertSign'      :'false',
                                          'cRLSign'          :'false',
                                          'encipherOnly'     :'false',
                                          'decipherOnly'     :'false'
                                          },
                                 'type':{'type':'dict'},
                                 'extension':'notpresent'
                                 },
            OPTION_EXT_EXTENDEDKEYUSAGE:{'value':{'serverAuth'     :'false',
                                                  'clientAuth'     :'false',
                                                  'codeSigning'    :'false',
                                                  'emailProtection':'false',
                                                  'timeStamping'   :'false',
                                                  'msCodeInd'      :'false',
                                                  'msCodeCom'      :'false',
                                                  'encipherOnly'   :'false',
                                                  'msCTLSign'      :'false',
                                                  'msSGC'          :'false',
                                                  'msEFS'          :'false',
                                                  'nsSGC'          :'false'
                                                  },
                                         'type':{'type':'dict'},
                                         'extension':'notpresent'
                                         },
            OPTION_EXT_SUBJECTKEYIDENTIFIER:{'value':{'hash':'false',
                                                      'hexstring':'false'
                                                      },
                                             'type':{'type':'choice'},
                                             'extension':'notpresent'},
            OPTION_EXT_AUTHORITYKEYIDENTIFIER:{'value':{'keyid':{'value':'','present':'false','type':{'type':'enum','vals':['always']}},
                                                        'issuer':{'value':'','present':'false','type':{'type':'enum','vals':['always']}}
                                                        },
                                               'type':{'type':'dict'},
                                               'extension':'notpresent'},
            OPTION_EXT_SUBJECTALTNAME:{'value':{'email'   :{'value':'','present':'false','type':{'type':'string','reserved_words':['copy',]}},
                                                'URI'     :{'value':'','present':'false','type':{'type':'string'}},
                                                'DNS'     :{'value':'','present':'false','type':{'type':'string'}},
                                                'RID'     :{'value':'','present':'false','type':{'type':'string'}},
                                                'IP'      :{'value':'','present':'false','type':{'type':'string'}}
                                                },
                                       'type':{'type':'dict'},
                                       'extension':'notpresent'},
            OPTION_EXT_ISSUERALTNAME:{'value':{'issuer'   :{'value':'','present':'false','type':{'type':'enum','vals':['copy']}},
                                               'URI'      :{'value':'','present':'false','type':{'type':'string'}},
                                               'DNS'      :{'value':'','present':'false','type':{'type':'string'}},
                                               'RID'      :{'value':'','present':'false','type':{'type':'string'}},
                                               'IP'       :{'value':'','present':'false','type':{'type':'string'}}
                                               },
                                      'type':{'type':'dict'},
                                      'extension':'notpresent'},
            OPTION_EXT_NSCERTTYPE:{'value':{'client'    :'false',
                                            'server'    :'false',
                                            'email'     :'false',
                                            'objsign'   :'false',
                                            'reserved'  :'false',
                                            'sslCA'     :'false',
                                            'emailCA'   :'false',
                                            'objCA'     :'false'
                                            },
                                   'type':{'type':'dict'},
                                   'extension':'notpresent'},
            OPTION_EXT_NSCOMMENT:{'value':'',
                                  'type':{'type':'string'},
                                  'extension':'notpresent'},
            OPTION_EXT_NSCAPOLICYURL:{'value':'',
                                      'type':{'type':'string'},
                                      'extension':'notpresent'},
            OPTION_EXT_NSCAREVOCATIONURL:{'value':'',
                                          'type':{'type':'string'},
                                          'extension':'notpresent'},
            OPTION_EXT_NSRENEWALURL:{'value':'',
                                     'type':{'type':'string'},
                                     'extension':'notpresent'},
            OPTION_EXT_CRLDISTRIBUTIONPOINTS:{'value':{'email'   :{'value':'','present':'false','type':{'type':'string'}},
                                                       'URI'     :{'value':'','present':'false','type':{'type':'string'}},
                                                       'DNS'     :{'value':'','present':'false','type':{'type':'string'}},
                                                       'RID'     :{'value':'','present':'false','type':{'type':'string'}},
                                                       'IP'      :{'value':'','present':'false','type':{'type':'string'}}
                                                       },
                                              'type':{'type':'dict'},
                                              'extension':'notpresent'},
            OPTION_EXT_CERTIFICATEPOLICIES:{'value':{'ID':{'value':'','present':'false','type':{'type':'string'}},
                                                     'CPS':{'value':'','present':'false','type':{'type':'string'}},
                                                     'userNotices':{'value':'','present':'false','type':{'type':'string'}}
                                                     },
                                            'type':{'type':'dict'},
                                            'extension':'notpresent'}
            }


def GetPolicy(dir,name,policy=0):

    if not policy:
        conf = OpensslConfFile()
        conf.readfp(os.path.join(dir,CA_EXTTEMPLATES_FILE_DIR,"%s.osslcnf" % name))
        section_ext = SECTION_EXTENSIONS
        section_pol = POLICY_SECTION
    else:
        conf = OpensslReadConfFile(dir)
        section_ext = "%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,name)
        section_pol = "%s_%s" % (PREAMBLE_POLICY_SECTION,name)

    return Policy2Dict(name,conf,section_ext,section_pol)

def Policy2Dict(name,conf,section_ext,section_pol):

    dict = NewEmptyPolicy()

    dict['name'] = name

    FillDictByExtension(dict,OPTION_NOEXT_VALIDITY,conf,section_pol,OPTION_NOEXT_VALIDITY)
    FillDictByExtension(dict,OPTION_NOEXT_BITSDOWN,conf,section_pol,OPTION_NOEXT_BITSDOWN)
    FillDictByExtension(dict,OPTION_NOEXT_BITSUP,conf,section_pol,OPTION_NOEXT_BITSUP)
    
    FillDictByExtension(dict,OPTION_EXT_BASICCONSTRAINTS,conf,section_ext,OPTION_EXT_BASICCONSTRAINTS)
    FillDictByExtension(dict,OPTION_EXT_KEYUSAGE,conf,section_ext,OPTION_EXT_KEYUSAGE)
    FillDictByExtension(dict,OPTION_EXT_EXTENDEDKEYUSAGE,conf,section_ext,OPTION_EXT_EXTENDEDKEYUSAGE)
    FillDictByExtension(dict,OPTION_EXT_SUBJECTKEYIDENTIFIER,conf,section_ext,OPTION_EXT_SUBJECTKEYIDENTIFIER)
    FillDictByExtension(dict,OPTION_EXT_AUTHORITYKEYIDENTIFIER,conf,section_ext,OPTION_EXT_AUTHORITYKEYIDENTIFIER)
    FillDictByExtension(dict,OPTION_EXT_SUBJECTALTNAME,conf,section_ext,OPTION_EXT_SUBJECTALTNAME)
    FillDictByExtension(dict,OPTION_EXT_ISSUERALTNAME,conf,section_ext,OPTION_EXT_ISSUERALTNAME)
    FillDictByExtension(dict,OPTION_EXT_NSCERTTYPE,conf,section_ext,OPTION_EXT_NSCERTTYPE)
    FillDictByExtension(dict,OPTION_EXT_NSCOMMENT,conf,section_ext,OPTION_EXT_NSCOMMENT)
    FillDictByExtension(dict,OPTION_EXT_NSCAPOLICYURL,conf,section_ext,OPTION_EXT_NSCAPOLICYURL)
    FillDictByExtension(dict,OPTION_EXT_NSCAREVOCATIONURL,conf,section_ext,OPTION_EXT_NSCAREVOCATIONURL)
    FillDictByExtension(dict,OPTION_EXT_NSRENEWALURL,conf,section_ext,OPTION_EXT_NSRENEWALURL)
    FillDictByExtension(dict,OPTION_EXT_CRLDISTRIBUTIONPOINTS,conf,section_ext,OPTION_EXT_CRLDISTRIBUTIONPOINTS)

    return dict

def Dict2Policy(dict,conf,section_ext,section_pol):
    
    WriteExtensionFromDict(dict,OPTION_NOEXT_VALIDITY,conf,section_pol,OPTION_NOEXT_VALIDITY)
    WriteExtensionFromDict(dict,OPTION_NOEXT_BITSDOWN,conf,section_pol,OPTION_NOEXT_BITSDOWN)
    WriteExtensionFromDict(dict,OPTION_NOEXT_BITSUP,conf,section_pol,OPTION_NOEXT_BITSUP)

    WriteExtensionFromDict(dict,OPTION_EXT_BASICCONSTRAINTS,conf,section_ext,OPTION_EXT_BASICCONSTRAINTS)
    WriteExtensionFromDict(dict,OPTION_EXT_KEYUSAGE,conf,section_ext,OPTION_EXT_KEYUSAGE)
    WriteExtensionFromDict(dict,OPTION_EXT_EXTENDEDKEYUSAGE,conf,section_ext,OPTION_EXT_EXTENDEDKEYUSAGE)
    WriteExtensionFromDict(dict,OPTION_EXT_SUBJECTKEYIDENTIFIER,conf,section_ext,OPTION_EXT_SUBJECTKEYIDENTIFIER)
    WriteExtensionFromDict(dict,OPTION_EXT_AUTHORITYKEYIDENTIFIER,conf,section_ext,OPTION_EXT_AUTHORITYKEYIDENTIFIER)
    WriteExtensionFromDict(dict,OPTION_EXT_SUBJECTALTNAME,conf,section_ext,OPTION_EXT_SUBJECTALTNAME)
    WriteExtensionFromDict(dict,OPTION_EXT_ISSUERALTNAME,conf,section_ext,OPTION_EXT_ISSUERALTNAME)
    WriteExtensionFromDict(dict,OPTION_EXT_NSCERTTYPE,conf,section_ext,OPTION_EXT_NSCERTTYPE)
    WriteExtensionFromDict(dict,OPTION_EXT_NSCOMMENT,conf,section_ext,OPTION_EXT_NSCOMMENT)
    WriteExtensionFromDict(dict,OPTION_EXT_NSCAPOLICYURL,conf,section_ext,OPTION_EXT_NSCAPOLICYURL)
    WriteExtensionFromDict(dict,OPTION_EXT_NSCAREVOCATIONURL,conf,section_ext,OPTION_EXT_NSCAREVOCATIONURL)
    WriteExtensionFromDict(dict,OPTION_EXT_NSRENEWALURL,conf,section_ext,OPTION_EXT_NSRENEWALURL)
    WriteExtensionFromDict(dict,OPTION_EXT_CRLDISTRIBUTIONPOINTS,conf,section_ext,OPTION_EXT_CRLDISTRIBUTIONPOINTS)

def AddExtensionTemplate(dir,filepath,name):
    shutil.copyfile(filepath,os.path.join(dir,CA_EXTTEMPLATES_FILE_DIR,"%s.osslcnf" % name))

def EnablePolicy(dir,policy):
    conf = OpensslReadConfFile(dir)
    pol_enabled  = EnabledPolicyList(conf)
    pol_disabled = DisabledPolicyList(conf)

    if policy not in pol_disabled:
        raise "OpensslError","Policy is already enabled or does not exist"
    
    pol_disabled.remove(policy)
    pol_enabled.append(policy)
    
    
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES,string.join(pol_enabled,','))
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,string.join(pol_disabled,','))
    OpensslWriteConfFile(dir,conf)

def DisablePolicy(dir,policy):
    conf = OpensslReadConfFile(dir)
    pol_enabled  = EnabledPolicyList(conf)
    pol_disabled = DisabledPolicyList(conf)

    if policy not in pol_enabled:
        raise "OpensslError","Policy is already enabled or does not exist"
    
    pol_enabled.remove(policy)
    pol_disabled.append(policy)
    
    
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES,string.join(pol_enabled,','))
    conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,string.join(pol_disabled,','))
    OpensslWriteConfFile(dir,conf)

def DeletePolicy(dir,policy):
    conf = OpensslReadConfFile(dir)
    pol_enabled  = EnabledPolicyList(conf)
    pol_disabled = DisabledPolicyList(conf)

    if policy in pol_enabled:
        pol_enabled.remove(policy)
        conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABENABLEDPOLICIES,string.join(pol_enabled,','))
    elif policy in pol_disabled:
        pol_disabled.remove(policy)
        conf.set(SECTION_CACONF,OPTION_CACONF_SECCLABDISABLEDPOLICIES,string.join(pol_disabled,','))
    else:
        raise "OpensslError","Policy does not exist"

    conf.remove_section("%s_%s" % (PREAMBLE_EXTENSIONS_SECTION,policy))
    conf.remove_section("%s_%s" % (PREAMBLE_POLICY_SECTION,policy))
    
    OpensslWriteConfFile(dir,conf)

def Certpem2Certder(pem):
    command = OPENSSL_BIN + ' x509 -inform pem -outform der'
    rc,output,error = ExecuteOpensslCommand(command,pem)
    if rc:
        raise "OpensslError",error
    return output

def Certder2Certpem(der):
    command = OPENSSL_BIN + ' x509 -inform der -outform pem'
    rc,output,error = ExecuteOpensslCommand(command,der)
    if rc:
        raise "OpensslError",error
    return output

def VerifyPolicyFromRequest(dir,spkac,filename,policy_name):
    try:
        policy = GetPolicy(dir,policy_name,1)
    except:
        raise "Error getting policy info %s" % policy_name,"(%s): %s" % (sys.exc_type,sys.exc_value)
    conf = OpensslReadConfFile(dir)
    list_pol = EnabledPolicyList(conf)
    if policy_name not in list_pol:
        raise "OpensslError","Policy is not enabled"
    if spkac:
        #command = OPENSSL_BIN + ' spkac -in "%s"|grep "Modulus"' % (filename)
        command = OPENSSL_BIN + ' spkac -in "%s"' % (filename)
    else:
        #command = OPENSSL_BIN + ' req -in "%s" -text|grep "Modulus"' % filename
        command = OPENSSL_BIN + ' req -in "%s" -text' % filename
    rc,output,error = ExecuteOpensslCommand(command,'')
    if rc:
        raise "OpensslError",error

    modulus = None

    for line in output.split("\n"):
        line = line.strip()
        if line[:7] == 'Modulus':
            modulus = line
            break
    values = string.split(modulus,"Modulus")
    # values = string.split(output,"Modulus")
    if len(values) < 2:
        raise "OpensslError","Error getting info from SPKAC"

    values = values[1].strip().split(' ')

    bits = int(values[0].strip()[1:])

    lbits = policy[OPTION_NOEXT_BITSDOWN]['value']
    hbits = policy[OPTION_NOEXT_BITSUP]['value']
    if lbits != 0:
        if bits < lbits:
            raise "OpensslError","Request key length not valid for policy (%d < %d)" % (bits,lbits)
    if hbits != 0:
        if bits > hbits:
            raise "OpensslError","Request key length not valid for policy (%d > %d)" % (bits,hbits)
            
    return policy[OPTION_NOEXT_VALIDITY]['value']


def CertifyRequest(dir,password,spkac,filename,policy):
    # Returns the certificate in PEM mode
    days = VerifyPolicyFromRequest(dir,spkac,filename,policy)
    if days == 0:
        days_str = ""
    else:
        days_str = '-days "%d"' % days
        
    if spkac == 0:
        input_str = '-in "%s"' % filename
    else:
        input_str = '-spkac "%s"' % filename
    command = OPENSSL_BIN + ' ca -config "%s" -passin stdin %s -policy policy_any -batch -notext %s -extensions "%s_%s"' % (os.path.join(dir,"openssl.conf"),input_str,days_str,PREAMBLE_EXTENSIONS_SECTION,policy)

    rc,output,error = ExecuteOpensslCommand(command,password)
    if rc:
        raise "OpensslError",error

    return output

def GetCertificateList(dir,password,fromIndex,count,status_req):
    certList = []
    f = open(os.path.join(dir,"index.txt"),"r")

    if len(status_req) == 0:
        status_list = ['R','V','E']
    else:
        status_list = []
        if 'r' in status_req:
            status_list.append('R')
        if 'v' in status_req:
            status_list.append('V')
        if 'e' in status_req:
            status_list.append('E')

    lineCounter = 0
    linesReaded = 0
    while linesReaded < count:
        s = f.readline()
        if len(s) == 0:
            break
        fields = string.split(s,'\t',6)
        if fields[0] not in status_list:
            continue

        if lineCounter < fromIndex:
            lineCounter = lineCounter + 1
        else:
        
            certList.append({
                'status': fields[0],
                'valid_from':fields[1],
                'not_valid_from':fields[2],
                'serial':fields[3],
                'dn':fields[5]
                })
            linesReaded = linesReaded + 1
    f.close()
    return certList

def GetIssuedCertificateInfo(dir,serial):
    command = OPENSSL_BIN + ' x509 -in "%s" -text -noout' % os.path.join(dir,CA_DIR_CERTS,"%s.pem" % serial)
    rc,output,error = ExecuteOpensslCommand(command,'')
    if rc:
        raise "OpensslError",error
    
    return output

def RevokeIssuedCert(dir,password,serial,reason,time,instruction):

    if reason == 'unspecified':
        param = '-crl_reason "%s"' % reason
    elif reason == 'keyCompromise':
        param = '-crl_reason "%s" -crl_compromise "%s"' % (reason,time)
    elif reason == 'CACompromise':
        param = '-crl_reason "%s" -crlCA_compromise' % (reason,time)
    elif reason == 'affiliationChanged':
        param = '-crl_reason "%s"' % reason
    elif reason == 'superseded':
        param = '-crl_reason "%s"' % reason
    elif reason == 'cessationOfOperation':
        param = '-crl_reason "%s"' % reason
    elif reason == 'certificateHold':
        param = '-crl_reason "%s" -crl_hold "%s"' % (reason,instruction)
    elif reason == 'removeFromCRL':
        param = '-crl_reason "%s"' % reason
    else:
        raise "OpensslError","revoke reason not valid (%s)" % reason
    
    command = OPENSSL_BIN + ' ca -config "%s" -passin stdin -revoke "%s" %s' % (os.path.join(dir,"openssl.conf"),os.path.join(dir,CA_DIR_CERTS,"%s.pem" % serial),param)
    
    rc,output,error = ExecuteOpensslCommand(command,password)
    if rc:
        raise "OpensslError",error

    return output


def CheckCAPassword(dir,password):
    command = OPENSSL_BIN + ' rsa -passin stdin -noout -in "%s"' % os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY)
    
    rc,output,error = ExecuteOpensslCommand(command,password)
    if rc:
        command = OPENSSL_BIN + ' dsa -passin stdin -noout -in "%s"' % os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY)
    
        rc,output,error = ExecuteOpensslCommand(command,password)
        if rc:
            return 0

    return 1

def GetCARequest(dir,password,x509):
    info_cert = OpensslGetInfoCert(os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT))
    if x509:
        x509_str = "-x509"
    else:
        x509_str = ""
    command = OPENSSL_BIN + ' req -new %s -key "%s" -passin stdin -subj "%s"'  % (x509_str,os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CAKEY),info_cert['subject'])
    rc,output,error = ExecuteOpensslCommand(command,"%s" % password)
    if rc:
        raise "OpensslError",error

    return output

def GetInfoFromSpkac(spkac):
    command = OPENSSL_BIN + ' spkac'
    rc,output,error = ExecuteOpensslCommand(command,spkac)
    if rc:
        return "DECODE SPKAC ERROR (%s)" % error
    return output

def GetInfoFromPkcs10(pkcs10):
    command = OPENSSL_BIN + ' req -text -noout'
    rc,output,error = ExecuteOpensslCommand(command,pkcs10)
    if rc:
        return "DECODE PKCS10 ERROR (%s)" % error
    return output
    


def GetCertificateInfo(cert):
    command = OPENSSL_BIN + ' x509 -inform der -text -noout'
    rc,output,error = ExecuteOpensslCommand(command,cert)
    if rc:
        raise "OpensslError",error
    
    return output

def GetCrlInfo(dir):
    if os.path.isfile(os.path.join(dir,CA_DIR_CRL,CA_FILE_CRL)):
      command = OPENSSL_BIN + ' crl -in "%s" -lastupdate -nextupdate -noout' % os.path.join(dir,CA_DIR_CRL,CA_FILE_CRL)
      rc,output,error = ExecuteOpensslCommand(command,"")
      if rc:
          return "GET CRL INFO ERROR (%s)" % error

      d = {}
      for line in string.split(output,"\n"):
          if line.find("=") != -1:
              token,value = line.split("=",2)
              s_t = time.strptime(value,"%b %d %H:%M:%S %Y %Z")
              d[token] = s_t
        
      return d
    else:
      return {}

def GetCrlExtendedInfo(dir):
    command = OPENSSL_BIN + ' crl -in "%s" -text' % os.path.join(dir,CA_DIR_CRL,CA_FILE_CRL)
    rc,output,error = ExecuteOpensslCommand(command,"")
    if rc:
        return "GET CRL INFO ERROR (%s)" % error

    return output

def GenerateNewCrl(dir,passwd):
    command = OPENSSL_BIN + ' ca -passin stdin -config "%s" -gencrl -out "%s"' % (os.path.join(dir,"openssl.conf"),os.path.join(dir,CA_DIR_CRL,CA_FILE_CRL))
    input = "%s\n" % passwd
    rc,output,error = ExecuteOpensslCommand(command,input)
    if rc:
        raise "OpensslError",error

    return output




def GetP7CAOwnCertificate(dir,type="pem",withPath=None):
    if type=='pem':
        outform = 'pem'
    else:
        outform = 'der'
    filename = os.path.join(dir,CA_DIR_PRIVATE_OWNCERTS,CA_FILE_CACERT)
    command = OPENSSL_BIN + ' crl2pkcs7 -certfile "%s" -nocrl -outform %s' % (filename,outform)
    rc,output,error = ExecuteOpensslCommand(command,"")
    if rc:
        raise "OpensslError",error

    return output

def GetCRL(dir,type="pem"):
    filename = os.path.join(dir,CA_DIR_CRL,CA_FILE_CRL)
    if type == 'pem':
        f = open(filename,"r")
        crl_pem = f.read()
        f.close()
        return crl_pem
    else:
        command = OPENSSL_BIN + ' crl -in "%s" -outform der' % filename
        rc,output,error = ExecuteOpensslCommand(command,"")
        if rc:
            raise "OpensslError",error
        
        return output

def CrlSwitchOutput(crl,from_type):
    if from_type == 'pem':
        inform = 'pem'
        outform = 'der'
    else:
        inform = 'der'
        outform = 'pem'
    
    command = OPENSSL_BIN + ' crl -inform %s -outform %s' % (inform,outform)
    rc,output,error = ExecuteOpensslCommand(command,crl)
    if rc:
        raise "OpensslError", error

    return output        

def P7SwitchOutput(p7,from_type):
    if from_type == 'pem':
        inform = 'pem'
        outform = 'der'
    else:
        inform = 'der'
        outform = 'pem'
    
    command = OPENSSL_BIN + ' pkcs7 -inform %s -outform %s' % (inform,outform)
    rc,output,error = ExecuteOpensslCommand(command,p7)
    if rc:
        raise "OpensslError", error

    return output     

def SetCRLGenPeriod(dir,crl_days=None,crl_hours=None):
    conf = OpensslReadConfFile(dir)

    if crl_days is not None:
        conf.set(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLDAYS,crl_days)
        conf.remove_option(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLHOURS)
    else:
        conf.set(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLHOURS,crl_hours)
        conf.remove_option(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLDAYS)

    OpensslWriteConfFile(dir,conf)

def GetCRLGenPeriod(dir):
    conf = OpensslReadConfFile(dir)

    if conf.has_option(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLDAYS):
        value_type = "crl_days"
        value = conf.get(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLDAYS)
    else:
        value_type = "crl_hours"
        value = conf.get(SECTION_CACONF,OPTION_CACONF_DEFAULTCRLHOURS)

    return {'type':value_type,'value':value}
